import { JSONObject } from "kuzzle-sdk";
export declare class KuzzleDebugger {
    private inspector;
    private debuggerStatus;
    /**
     * Map<eventName, Set<connectionId>>
     */
    private events;
    private httpWsProtocol?;
    init(): Promise<void>;
    registerAsks(): Promise<void>;
    /**
     * Connect the debugger
     */
    enable(): Promise<void>;
    /**
     * Disconnect the debugger and clears all the events listeners
     */
    disable(): Promise<void>;
    /**
     * Trigger action from debugger directly following the Chrome Debug Protocol
     * See: https://chromedevtools.github.io/devtools-protocol/v8/
     */
    post(method: string, params?: JSONObject): Promise<JSONObject>;
    /**
     * Make the websocket connection listen and receive events from Chrome Debug Protocol
     * See events from: https://chromedevtools.github.io/devtools-protocol/v8/
     */
    addListener(event: string, connectionId: string): Promise<void>;
    /**
     * Remove the websocket connection from the events" listeners
     */
    removeListener(event: string, connectionId: string): Promise<void>;
    /**
     * Execute a method using the Chrome Debug Protocol
     * @param method Chrome Debug Protocol method to execute
     * @param params
     * @returns
     */
    private inspectorPost;
    /**
     * Sends a direct notification to a websocket connection without having to listen to a specific room
     */
    private notifyConnection;
    private notifyGlobalListeners;
}
